/**
 * Interface of the server (auto-generated code)
 */
import {
	HTTPMimeType,
	HTTPRequestModel,
	RESTCallbacks,
	RESTConfig,
	RESTRequestJson,
	RESTRequestVoid,
} from "../rest-tools";

import { z as zod } from "zod"
import {
	Long,
	Type,
	TypeWrite,
	UUID,
	ZodType,
	isType,
} from "../model";

export namespace TypeResource {

	/**
	 * Get a specific Type with his ID
	 */
	export function get({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
	}): Promise<Type> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/{id}",
				requestType: HTTPRequestModel.GET,
				contentType: HTTPMimeType.JSON,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
		}, isType);
	};

	export const ZodGetsTypeReturn = zod.array(ZodType);
	export type GetsTypeReturn = zod.infer<typeof ZodGetsTypeReturn>;
	
	export function isGetsTypeReturn(data: any): data is GetsTypeReturn {
		try {
			ZodGetsTypeReturn.parse(data);
			return true;
		} catch (e: any) {
			console.log(`Fail to parse data type='ZodGetsTypeReturn' error=${e}`);
			return false;
		}
	}

	/**
	 * Get all Type
	 */
	export function gets({
			restConfig,
		}: {
		restConfig: RESTConfig,
	}): Promise<GetsTypeReturn> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/",
				requestType: HTTPRequestModel.GET,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
		}, isGetsTypeReturn);
	};
	/**
	 * Modify a specific Type
	 */
	export function patch({
			restConfig,
			params,
			data,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
		data: TypeWrite,
	}): Promise<Type> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/{id}",
				requestType: HTTPRequestModel.PATCH,
				contentType: HTTPMimeType.JSON,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
			data,
		}, isType);
	};
	/**
	 * Create a new Type
	 */
	export function post({
			restConfig,
			data,
		}: {
		restConfig: RESTConfig,
		data: TypeWrite,
	}): Promise<Type> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/",
				requestType: HTTPRequestModel.POST,
				contentType: HTTPMimeType.JSON,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			data,
		}, isType);
	};
	/**
	 * Remove a specific Type
	 */
	export function remove({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
	}): Promise<void> {
		return RESTRequestVoid({
			restModel: {
				endPoint: "/type/{id}",
				requestType: HTTPRequestModel.DELETE,
				contentType: HTTPMimeType.TEXT_PLAIN,
			},
			restConfig,
			params,
		});
	};
	/**
	 * Remove a specific cover of a type
	 */
	export function removeCover({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			coverId: UUID,
			id: Long,
		},
	}): Promise<Type> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/{id}/cover/{coverId}",
				requestType: HTTPRequestModel.DELETE,
				contentType: HTTPMimeType.TEXT_PLAIN,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
		}, isType);
	};
	/**
	 * Upload a new season cover Type
	 */
	export function uploadCover({
			restConfig,
			params,
			data,
			callbacks,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
		data: {
			file: File,
		},
		callbacks?: RESTCallbacks,
	}): Promise<Type> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/type/{id}/cover",
				requestType: HTTPRequestModel.POST,
				contentType: HTTPMimeType.MULTIPART,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
			data,
			callbacks,
		}, isType);
	};
}
